@@ -35,15 +35,11 @@
 #include "FbTk/RefCount.hh"
 #include "FbTk/KeyUtil.hh"
 #include "FbTk/CommandParser.hh"
+#include "FbTk/LogicCommands.hh"
 #include "FbTk/I18n.hh"
 #include "FbTk/AutoReloadHelper.hh"
 #include "FbTk/STLUtil.hh"
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif // HAVE_CONFIG_H
-
-
 #ifdef HAVE_CCTYPE
   #include <cctype>
 #else
@@ -109,6 +105,11 @@ using FbTk::STLUtil::destroyAndClearSecond;
 
 namespace {
 
+// enforces the linking of FbTk/LogicCommands
+FbTk::Command<void>* link_helper = FbTk::IfCommand::parse("", "", false);
+
+
+
 // candidate for FbTk::StringUtil ?
 int extractKeyFromString(const std::string& in, const char* start_pattern, unsigned int& key) {
 
@@ -146,7 +147,7 @@ public:
         // t_key ctor sets context_ of 0 to GLOBAL, so we must here too
         context_ = context_ ? context_ : GLOBAL;
         keylist_t::iterator it = keylist.begin(), it_end = keylist.end();
-        for (; it != it_end; it++) {
+        for (; it != it_end; ++it) {
             if (*it && (*it)->type == type_ && (*it)->key == key_ &&
                 ((*it)->context & context_) > 0 &&
                 isdouble_ == (*it)->isdouble && (*it)->mod ==
@@ -405,6 +406,8 @@ bool Keys::addBinding(const string &linebuffer) {
                 context |= ON_LEFTGRIP;
             else if (arg == "onrightgrip")
                 context |= ON_RIGHTGRIP;
+            else if (arg == "ontab")
+                context |= ON_TAB;
             else if (arg == "double")
                 isdouble = true;
             else if (arg != "none") {
@@ -487,7 +490,6 @@ bool Keys::addBinding(const string &linebuffer) {
                 }
                 mod = 0;
                 key = 0;
-                type = 0;
                 context = 0;
                 isdouble = false;
             }
@@ -573,11 +575,6 @@ bool Keys::doAction(int type, unsigned int mods, unsigned int key,
         return false;
     }
 
-    // if focus changes, windows will get NotifyWhileGrabbed,
-    // which they tend to ignore
-    if (type == KeyPress)
-        XUngrabKeyboard(Fluxbox::instance()->display(), CurrentTime);
-
     WinClient *old = WindowCmd<void>::client();
     WindowCmd<void>::setClient(current);
     temp_key->m_command->execute();
