@@ -53,12 +53,14 @@
 #include "FbTk/CommandParser.hh"
 #include "AtomHandler.hh"
 #include "HeadArea.hh"
+#include "RectangleUtil.hh"
 #include "FbCommands.hh"
+#ifdef USE_SYSTRAY
 #include "SystemTray.hh"
+#endif
 #include "Debug.hh"
 
 #include "FbTk/I18n.hh"
-#include "FbTk/Subject.hh"
 #include "FbTk/FbWindow.hh"
 #include "FbTk/SimpleCommand.hh"
 #include "FbTk/MultLayers.hh"
@@ -75,22 +77,13 @@
 #include "FbTk/KeyUtil.hh"
 #include "FbTk/Util.hh"
 
-//use GNU extensions
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE
-#endif // _GNU_SOURCE
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif // HAVE_CONFIG_H
-
-#ifdef SLIT
+#ifdef USE_SLIT
 #include "Slit.hh"
 #include "SlitClient.hh"
 #else
 // fill it in
 class Slit {};
-#endif // SLIT
+#endif // USE_SLIT
 
 #ifdef USE_TOOLBAR
 #include "Toolbar.hh"
@@ -132,7 +125,7 @@ extern  "C" {
 }
 #endif // XINERAMA
 
-#ifdef HAVE_RANDR
+#if defined(HAVE_RANDR) || defined(HAVE_RANDR1_2)
 #include <X11/extensions/Xrandr.h>
 #endif // HAVE_RANDR
 
@@ -229,6 +222,22 @@ const TabPlacementString placement_strings[] = {
     { FbWinFrame::RIGHTTOP, "RightTop" }
 };
 
+Atom atom_fbcmd = 0;
+Atom atom_wm_check = 0;
+Atom atom_net_desktop = 0;
+Atom atom_utf8_string = 0;
+Atom atom_kde_systray = 0;
+Atom atom_kwm1 = 0;
+
+void initAtoms(Display* dpy) {
+    atom_wm_check = XInternAtom(dpy, "_NET_SUPPORTING_WM_CHECK", False);
+    atom_net_desktop = XInternAtom(dpy, "_NET_CURRENT_DESKTOP", False);
+    atom_fbcmd = XInternAtom(dpy, "_FLUXBOX_ACTION", False);
+    atom_utf8_string = XInternAtom(dpy, "UTF8_STRING", False);
+    atom_kde_systray = XInternAtom(dpy, "_KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR", False);
+    atom_kwm1 = XInternAtom(dpy, "KWM_DOCKWINDOW", False);
+}
+
 
 } // end anonymous namespace
 
@@ -300,7 +309,6 @@ BScreen::BScreen(FbTk::ResourceManager &rm,
                  const string &screenname,
                  const string &altscreenname,
                  int scrn, int num_layers) :
-    m_reconfigure_sig(*this), // reconfigure signal
     m_layermanager(num_layers),
     m_image_control(0),
     m_focused_windowtheme(new FbWinFrameTheme(scrn, ".focus", ".Focus")),
@@ -316,8 +324,7 @@ BScreen::BScreen(FbTk::ResourceManager &rm,
     m_geom_window(new OSDWindow(m_root_window, *this, *m_focused_windowtheme)),
     m_pos_window(new OSDWindow(m_root_window, *this, *m_focused_windowtheme)),
     m_tooltip_window(new TooltipWindow(m_root_window, *this, *m_focused_windowtheme)),
-    m_dummy_window(scrn, -1, -1, 1, 1, 0, true, false, CopyFromParent,
-                   InputOnly),
+    m_dummy_window(scrn, -1, -1, 1, 1, 0, true, false, CopyFromParent, InputOnly),
     resource(rm, screenname, altscreenname),
     m_resource_manager(rm),
     m_name(screenname),
@@ -330,8 +337,11 @@ BScreen::BScreen(FbTk::ResourceManager &rm,
     m_shutdown(false) {
 
 
-    Display *disp = m_root_window.display();
     Fluxbox *fluxbox = Fluxbox::instance();
+    Display *disp = fluxbox->display();
+
+    initAtoms(disp);
+
 
     // TODO fluxgen: check if this is the right place (it was not -lis)
     //
@@ -348,7 +358,7 @@ BScreen::BScreen(FbTk::ResourceManager &rm,
                               SubstructureRedirectMask | KeyPressMask | KeyReleaseMask |
                               ButtonPressMask | ButtonReleaseMask| SubstructureNotifyMask);
 
-    FbTk::App::instance()->sync(false);
+    fluxbox->sync(false);
 
     XSetErrorHandler((XErrorHandler) old);
 
@@ -361,7 +371,7 @@ BScreen::BScreen(FbTk::ResourceManager &rm,
 
     // we're going to manage the screen, so now add our pid
 #ifdef HAVE_GETPID
-    pid_t bpid = getpid();
+    unsigned long bpid = static_cast<unsigned long>(getpid());
 
     rootWindow().changeProperty(fluxbox->getFluxboxPidAtom(), XA_CARDINAL,
                                 sizeof(pid_t) * 8, PropModeReplace,
@@ -369,31 +379,34 @@ BScreen::BScreen(FbTk::ResourceManager &rm,
 #endif // HAVE_GETPID
 
     // check if we're the first EWMH compliant window manager on this screen
-    Atom wm_check = XInternAtom(disp, "_NET_SUPPORTING_WM_CHECK", False);
     Atom xa_ret_type;
     int ret_format;
     unsigned long ret_nitems, ret_bytes_after;
     unsigned char *ret_prop;
-    if (rootWindow().property(wm_check, 0l, 1l,
+    if (rootWindow().property(atom_wm_check, 0l, 1l,
             False, XA_WINDOW, &xa_ret_type, &ret_format, &ret_nitems,
-            &ret_bytes_after, &ret_prop) == Success) {
+            &ret_bytes_after, &ret_prop) ) {
         m_restart = (ret_prop != NULL);
         XFree(ret_prop);
     }
 
-#ifdef HAVE_RANDR
-    // setup RANDR for this screens root window
-    // we need to determine if we should use old randr select input function or not
-#ifdef X_RRScreenChangeSelectInput
-    // use old set randr event
-    XRRScreenChangeSelectInput(disp, rootWindow().window(), True);
-#else
-    XRRSelectInput(disp, rootWindow().window(),
-                   RRScreenChangeNotifyMask);
-#endif // X_RRScreenChangeSelectInput
 
+// setup RANDR for this screens root window
+#if defined(HAVE_RANDR)
+    int randr_mask = RRScreenChangeNotifyMask;
+# ifdef RRCrtcChangeNotifyMask
+    randr_mask |= RRCrtcChangeNotifyMask;
+# endif
+# ifdef RROutputChangeNotifyMask
+    randr_mask |= RROutputChangeNotifyMask;
+# endif
+# ifdef RROutputPropertyNotifyMask
+    randr_mask |= RROutputPropertyNotifyMask;
+# endif
+    XRRSelectInput(disp, rootWindow().window(), randr_mask);
 #endif // HAVE_RANDR
 
+
     _FB_USES_NLS;
 
 #ifdef DEBUG
@@ -407,7 +420,7 @@ BScreen::BScreen(FbTk::ResourceManager &rm,
 
     FbTk::EventManager *evm = FbTk::EventManager::instance();
     evm->add(*this, rootWindow());
-    Keys *keys = Fluxbox::instance()->keys();
+    Keys *keys = fluxbox->keys();
     if (keys)
         keys->registerWindow(rootWindow().window(), *this,
                              Keys::GLOBAL|Keys::ON_DESKTOP);
@@ -434,21 +447,14 @@ BScreen::BScreen(FbTk::ResourceManager &rm,
 
     m_menutheme->setDelay(*resource.menu_delay);
 
-    focusedWinFrameTheme()->reconfigSig().attach(this);// for geom window
+    m_tracker.join(focusedWinFrameTheme()->reconfigSig(),
+            FbTk::MemFun(*this, &BScreen::focusedWinFrameThemeReconfigured));
 
 
     renderGeomWindow();
     renderPosWindow();
     m_tooltip_window->setDelay(*resource.tooltip_delay);
 
-#if 0 // TODO: check if this is obsolete code!
-
-    //!! TODO: we shouldn't do this more than once, but since slit handles their
-    // own resources we must do this.
-    fluxbox->load_rc(*this);
-
-#endif
-
     // setup workspaces and workspace menu
     int nr_ws = *resource.workspaces;
     addWorkspace(); // at least one
@@ -475,22 +481,20 @@ BScreen::BScreen(FbTk::ResourceManager &rm,
     // check which desktop we should start on
     unsigned int first_desktop = 0;
     if (m_restart) {
-        Atom net_desktop = XInternAtom(disp, "_NET_CURRENT_DESKTOP", False);
-        if (rootWindow().property(net_desktop, 0l, 1l, 
-                False, XA_CARDINAL, &xa_ret_type, &ret_format, &ret_nitems,
-                &ret_bytes_after, &ret_prop) == Success) {
-            if (ret_prop && static_cast<unsigned int>(*ret_prop) < static_cast<unsigned int>(nr_ws))
-                first_desktop = static_cast<unsigned int>(*ret_prop);
-            XFree(ret_prop);
+        bool exists;
+        unsigned int ret=static_cast<unsigned int>(rootWindow().cardinalProperty(atom_net_desktop, &exists));
+        if (exists) {
+            if (ret < static_cast<unsigned int>(nr_ws))
+                first_desktop = ret;
         }
     }
 
     changeWorkspaceID(first_desktop);
 
-#ifdef SLIT
-    m_slit.reset(new Slit(*this, *layerManager().getLayer(Layer::DESKTOP),
+#ifdef USE_SLIT
+    m_slit.reset(new Slit(*this, *layerManager().getLayer(ResourceLayer::DESKTOP),
                  fluxbox->getSlitlistFilename().c_str()));
-#endif // SLIT
+#endif // USE_SLIT
 
     rm.unlock();
 
@@ -503,7 +507,9 @@ BScreen::~BScreen() {
 
     if (! managed)
         return;
-
+    
+    m_configmenu.reset(0);
+    
     m_toolbar.reset(0);
 
     FbTk::EventManager *evm = FbTk::EventManager::instance();
@@ -572,7 +578,12 @@ BScreen::~BScreen() {
 
     // slit must be destroyed before headAreas (Struts)
     m_slit.reset(0);
+    
 
+    delete m_rootmenu.release();
+    delete m_workspacemenu.release();
+    delete m_windowmenu.release();
+    
     // TODO fluxgen: check if this is the right place
     for (size_t i = 0; i < m_head_areas.size(); i++)
         delete m_head_areas[i];
@@ -590,7 +601,7 @@ void BScreen::initWindows() {
 
 #ifdef USE_TOOLBAR
     m_toolbar.reset(new Toolbar(*this,
-                                *layerManager().getLayer(::Layer::NORMAL)));
+                                *layerManager().getLayer(::ResourceLayer::NORMAL)));
 #endif // USE_TOOLBAR
 
     unsigned int nchild;
@@ -684,10 +695,10 @@ void BScreen::initWindows() {
     XFree(children);
 
     // now, show slit and toolbar
-#ifdef SLIT
+#ifdef USE_SLIT
     if (slit())
         slit()->show();
-#endif // SLIT
+#endif // USE_SLIT
 
 }
 
@@ -742,10 +753,7 @@ unsigned int BScreen::maxBottom(int head) const {
         return doFullMax() ? height() : height() - availableWorkspaceArea(head)->bottom();
 }
 
-void BScreen::update(FbTk::Subject *subj) {
-    // for now we're only listening to the theme sig, so no object check
-    // if another signal is added later, will need to differentiate here
-
+void BScreen::focusedWinFrameThemeReconfigured() {
     renderGeomWindow();
     renderPosWindow();
 
@@ -760,29 +768,29 @@ void BScreen::update(FbTk::Subject *subj) {
 }
 
 void BScreen::propertyNotify(Atom atom) {
-    static Atom fbcmd_atom = XInternAtom(FbTk::App::instance()->display(),
-                                         "_FLUXBOX_ACTION", False);
-    if (allowRemoteActions() && atom == fbcmd_atom) {
+
+    if (allowRemoteActions() && atom == atom_fbcmd) {
         Atom xa_ret_type;
         int ret_format;
         unsigned long ret_nitems, ret_bytes_after;
         char *str;
-        if (rootWindow().property(fbcmd_atom, 0l, 64l,
+        if (rootWindow().property(atom_fbcmd, 0l, 64l,
                 True, XA_STRING, &xa_ret_type, &ret_format, &ret_nitems,
                 &ret_bytes_after, (unsigned char **)&str) && str) {
 
             if (ret_bytes_after) {
                 XFree(str);
                 long len = 64 + (ret_bytes_after + 3)/4;
-                rootWindow().property(fbcmd_atom, 0l, len,
+                rootWindow().property(atom_fbcmd, 0l, len,
                     True, XA_STRING, &xa_ret_type, &ret_format, &ret_nitems,
                     &ret_bytes_after, (unsigned char **)&str);
             }
 
             static std::auto_ptr<FbTk::Command<void> > cmd(0);
             cmd.reset(FbTk::CommandParser<void>::instance().parse(str, false));
-            if (cmd.get())
+            if (cmd.get()) {
                 cmd->execute();
+            }
             XFree(str);
 
         }
@@ -792,12 +800,8 @@ void BScreen::propertyNotify(Atom atom) {
 }
 
 void BScreen::keyPressEvent(XKeyEvent &ke) {
-    if (Fluxbox::instance()->keys()->doAction(ke.type, ke.state, ke.keycode,
-                                              Keys::GLOBAL|Keys::ON_DESKTOP))
-        // re-grab keyboard, so we don't pass KeyRelease to clients
-        // also for catching invalid keys in the middle of keychains
-        FbTk::EventManager::instance()->grabKeyboard(rootWindow().window());
-
+    Fluxbox::instance()->keys()->doAction(ke.type, ke.state, ke.keycode,
+                                          Keys::GLOBAL|Keys::ON_DESKTOP);
 }
 
 void BScreen::keyReleaseEvent(XKeyEvent &ke) {
@@ -848,9 +852,8 @@ void BScreen::cycleFocus(int options, const ClientPattern *pat, bool reverse) {
 }
 
 FbMenu *BScreen::createMenu(const string &label) {
-    FbMenu *menu = new FbMenu(menuTheme(),
-                                  imageControl(),
-                                  *layerManager().getLayer(Layer::MENU));
+    FbTk::Layer* layer = layerManager().getLayer(ResourceLayer::MENU);
+    FbMenu *menu = new FbMenu(menuTheme(), imageControl(), *layer);
     if (!label.empty())
         menu->setLabel(label);
 
@@ -858,9 +861,8 @@ FbMenu *BScreen::createMenu(const string &label) {
 }
 
 FbMenu *BScreen::createToggleMenu(const string &label) {
-    FbMenu *menu = new ToggleMenu(menuTheme(),
-                                      imageControl(),
-                                      *layerManager().getLayer(Layer::MENU));
+    FbTk::Layer* layer = layerManager().getLayer(ResourceLayer::MENU);
+    FbMenu *menu = new ToggleMenu(menuTheme(), imageControl(), *layer);
     if (!label.empty())
         menu->setLabel(label);
 
@@ -913,7 +915,7 @@ void BScreen::reconfigure() {
 
     imageControl().cleanCache();
     // notify objects that the screen is reconfigured
-    m_reconfigure_sig.notify();
+    m_reconfigure_sig.emit(*this);
 
     // Reload style
     FbTk::ThemeManager::instance().load(fluxbox->getStyleFilename(),
@@ -1173,9 +1175,7 @@ bool BScreen::isKdeDockapp(Window client) const {
     unsigned long *data = 0, uljunk;
     Display *disp = FbTk::App::instance()->display();
     // Check if KDE v2.x dock applet
-    if (XGetWindowProperty(disp, client,
-                           XInternAtom(FbTk::App::instance()->display(),
-                                       "_KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR", False),
+    if (XGetWindowProperty(disp, client, atom_kde_systray,
                            0l, 1l, False,
                            XA_WINDOW, &ajunk, &ijunk, &uljunk,
                            &uljunk, (unsigned char **) &data) == Success) {
@@ -1188,11 +1188,9 @@ bool BScreen::isKdeDockapp(Window client) const {
 
     // Check if KDE v1.x dock applet
     if (!iskdedockapp) {
-        Atom kwm1 = XInternAtom(FbTk::App::instance()->display(),
-                                "KWM_DOCKWINDOW", False);
         if (XGetWindowProperty(disp, client,
-                               kwm1, 0l, 1l, False,
-                               kwm1, &ajunk, &ijunk, &uljunk,
+                               atom_kwm1, 0l, 1l, False,
+                               atom_kwm1, &ajunk, &ijunk, &uljunk,
                                &uljunk, (unsigned char **) &data) == Success && data) {
             iskdedockapp = (data && data[0] != 0);
             XFree((void *) data);
@@ -1210,15 +1208,15 @@ bool BScreen::addKdeDockapp(Window client) {
     FbTk::EventManager *evm = FbTk::EventManager::instance();
 
     AtomHandler* handler = 0;
-#if USE_TOOLBAR
+#if USE_SYSTRAY
     handler = Fluxbox::instance()->getAtomHandler(SystemTray::getNetSystemTrayAtom(screenNumber()));
 #endif
     if (handler == 0) {
-#ifdef SLIT
+#ifdef USE_SLIT
         if (slit() != 0 && slit()->acceptKdeDockapp())
             slit()->addClient(client);
         else
-#endif // SLIT
+#endif // USE_SLIT
             return false;
     } else {
         // this handler is a special case
@@ -1238,19 +1236,19 @@ bool BScreen::addKdeDockapp(Window client) {
 FluxboxWindow *BScreen::createWindow(Window client) {
     FbTk::App::instance()->sync(false);
 
-
     if (isKdeDockapp(client) && addKdeDockapp(client)) {
         return 0; // dont create a FluxboxWindow for this one
     }
 
     WinClient *winclient = new WinClient(client, *this);
 
-    if (winclient->initial_state == WithdrawnState) {
+    if (winclient->initial_state == WithdrawnState ||
+        winclient->getWMClassClass() == "DockApp") {
         delete winclient;
-#ifdef SLIT
+#ifdef USE_SLIT
         if (slit() && !isKdeDockapp(client))
             slit()->addClient(client);
-#endif // SLIT
+#endif // USE_SLIT
         return 0;
     }
 
@@ -1308,8 +1306,13 @@ FluxboxWindow *BScreen::createWindow(WinClient &client) {
     FluxboxWindow *win = new FluxboxWindow(client);
 
 #ifdef SLIT
-    if (win->initialState() == WithdrawnState && slit() != 0) {
-        slit()->addClient(client.window());
+    if (slit() != 0) {
+
+        if (win->initialState() == WithdrawnState) {
+            slit()->addClient(client.window());
+        } else if (client->getWMClassClass() == "DockApp") {
+            slit()->addClient(client.window());
+        }
     }
 #endif // SLIT
 
@@ -1468,7 +1471,8 @@ void BScreen::rereadWindowMenu() {
 
 void BScreen::addConfigMenu(const FbTk::FbString &label, FbTk::Menu &menu) {
     m_configmenu_list.push_back(make_pair(label, &menu));
-    setupConfigmenu(*m_configmenu.get());
+    if (m_configmenu.get())
+        setupConfigmenu(*m_configmenu.get());
 }
 
 void BScreen::removeConfigMenu(FbTk::Menu &menu) {
@@ -1479,7 +1483,7 @@ void BScreen::removeConfigMenu(FbTk::Menu &menu) {
     if (erase_it != m_configmenu_list.end())
         m_configmenu_list.erase(erase_it);
 
-    if (!isShuttingdown())
+    if (!isShuttingdown() && m_configmenu.get())
         setupConfigmenu(*m_configmenu.get());
 
 }
@@ -1489,6 +1493,55 @@ void BScreen::addManagedResource(FbTk::Resource_base *resource) {
     m_managed_resources.push_back(resource);
 }
 
+int BScreen::getGap(int head, const char type) {
+    return type == 'w' ? getXGap(head) : getYGap(head);
+}
+
+int BScreen::calRelativeSize(int head, int i, char type) {
+    // return floor(i * getGap(head, type) / 100 + 0.5);
+    return FbTk::RelCalcHelper::calPercentageValueOf(i, getGap(head, type));
+}
+int BScreen::calRelativeWidth(int head, int i) {
+    return calRelativeSize(head, i, 'w');
+}
+int BScreen::calRelativeHeight(int head, int i) {
+    return calRelativeSize(head, i, 'h');
+}
+
+int BScreen::calRelativePosition(int head, int i, char type) {
+    int max = type == 'w' ? maxLeft(head) : maxTop(head);
+    // return floor((i - min) / getGap(head, type)  * 100 + 0.5);
+    return FbTk::RelCalcHelper::calPercentageOf((i - max), getGap(head, type));
+}
+// returns a pixel, which is relative to the width of the screen
+// screen starts from 0, 1000 px width, if i is 10 then it should return 100
+int BScreen::calRelativePositionWidth(int head, int i) {
+    return calRelativePosition(head, i, 'w');
+}
+// returns a pixel, which is relative to the height of th escreen
+// screen starts from 0, 1000 px height, if i is 10 then it should return 100
+int BScreen::calRelativePositionHeight(int head, int i) {
+    return calRelativePosition(head, i, 'h');
+}
+
+int BScreen::calRelativeDimension(int head, int i, char type) {
+    // return floor(i / getGap(head, type) * 100 + 0.5);
+    return FbTk::RelCalcHelper::calPercentageOf(i, getGap(head, type));
+  }
+int BScreen::calRelativeDimensionWidth(int head, int i) {
+    return calRelativeDimension(head, i, 'w');
+}
+int BScreen::calRelativeDimensionHeight(int head, int i) {
+    return calRelativeDimension(head, i, 'h');
+}
+
+float BScreen::getXGap(int head) {
+    return maxRight(head) - maxLeft(head);
+}
+float BScreen::getYGap(int head) {
+    return maxBottom(head) - maxTop(head);
+}
+
 void BScreen::setupConfigmenu(FbTk::Menu &menu) {
     _FB_USES_NLS;
 
@@ -1552,9 +1605,20 @@ void BScreen::setupConfigmenu(FbTk::Menu &menu) {
             "Focus New Windows", "Focus newly created windows"),
             m_resource_manager.getResource<bool>(name() + ".focusNewWindows"),
             saverc_cmd));
+    } catch (FbTk::ResourceException & e) {
+        cerr<<e.what()<<endl;
+    }
+   
+#ifdef XINERAMA
+    try {
+        focus_menu->insert(new FbTk::BoolMenuItem(_FB_XTEXT(Configmenu, FocusSameHead,
+            "Keep Head", "Only revert focus on same head"),
+            m_resource_manager.getResource<bool>(name() + ".focusSameHead"),
+            saverc_cmd));
     } catch (FbTk::ResourceException e) {
         cerr<<e.what()<<endl;
     }
+#endif // XINERAMA
 
     _BOOLITEM(*focus_menu, Configmenu, AutoRaise,
               "Auto Raise", "Auto Raise windows on sloppy",
@@ -1803,21 +1867,21 @@ void BScreen::setLayer(FbTk::LayerItem &item, int layernum) {
 /**
  Goes to the workspace "right" of the current
 */
-void BScreen::nextWorkspace(const int delta) {
+void BScreen::nextWorkspace(int delta) {
     changeWorkspaceID( (currentWorkspaceID() + delta) % numberOfWorkspaces());
 }
 
 /**
  Goes to the workspace "left" of the current
 */
-void BScreen::prevWorkspace(const int delta) {
+void BScreen::prevWorkspace(int delta) {
     changeWorkspaceID( (static_cast<signed>(numberOfWorkspaces()) + currentWorkspaceID() - (delta % numberOfWorkspaces())) % numberOfWorkspaces());
 }
 
 /**
  Goes to the workspace "right" of the current
 */
-void BScreen::rightWorkspace(const int delta) {
+void BScreen::rightWorkspace(int delta) {
     if (currentWorkspaceID()+delta < numberOfWorkspaces())
         changeWorkspaceID(currentWorkspaceID()+delta);
 }
@@ -1825,7 +1889,7 @@ void BScreen::rightWorkspace(const int delta) {
 /**
  Goes to the workspace "left" of the current
 */
-void BScreen::leftWorkspace(const int delta) {
+void BScreen::leftWorkspace(int delta) {
     if (currentWorkspaceID() >= static_cast<unsigned int>(delta))
         changeWorkspaceID(currentWorkspaceID()-delta);
 }
@@ -1836,19 +1900,18 @@ void BScreen::renderGeomWindow() {
     char buf[256];
     _FB_USES_NLS;
 
-    sprintf(buf,
-            _FB_XTEXT(Screen, GeometrySpacing,
-            "W: %04d x H: %04d", "Representative maximum sized text for width and height dialog").c_str(),
-            0, 0);
+    const std::string msg = _FB_XTEXT(Screen, GeometrySpacing,
+            "W: %04d x H: %04d", "Representative maximum sized text for width and height dialog");
+    const int n = snprintf(buf, msg.size(), msg.c_str(), 0, 0);
 
-    FbTk::BiDiString label(buf);
-    m_geom_window->resize(label);
+    FbTk::BiDiString label(std::string(buf, n));
+    m_geom_window->resizeForText(label);
     m_geom_window->reconfigTheme();
 }
 
 
 void BScreen::renderPosWindow() {
-    m_pos_window->resize(FbTk::BiDiString("0:00000 x 0:00000"));
+    m_pos_window->resizeForText(FbTk::BiDiString("0:00000 x 0:00000"));
     m_pos_window->reconfigTheme();
 }
 
@@ -1950,10 +2013,10 @@ void BScreen::initXinerama() {
     m_xinerama_headinfo = new XineramaHeadInfo[number];
     m_xinerama_num_heads = number;
     for (int i=0; i < number; i++) {
-        m_xinerama_headinfo[i].x = screen_info[i].x_org;
-        m_xinerama_headinfo[i].y = screen_info[i].y_org;
-        m_xinerama_headinfo[i].width = screen_info[i].width;
-        m_xinerama_headinfo[i].height = screen_info[i].height;
+        m_xinerama_headinfo[i]._x = screen_info[i].x_org;
+        m_xinerama_headinfo[i]._y = screen_info[i].y_org;
+        m_xinerama_headinfo[i]._width = screen_info[i].width;
+        m_xinerama_headinfo[i]._height = screen_info[i].height;
     }
     XFree(screen_info);
 
@@ -1985,15 +2048,30 @@ void BScreen::clearHeads() {
     if (!hasXinerama()) return;
 
     for (Workspaces::iterator i = m_workspaces_list.begin();
-            i != m_workspaces_list.end(); i++) {
+            i != m_workspaces_list.end(); ++i) {
         for (Workspace::Windows::iterator win = (*i)->windowList().begin();
-                win != (*i)->windowList().end(); win++) {
+                win != (*i)->windowList().end(); ++win) {
 
-            int closest_head = getHead((*win)->fbWindow());
-            if (closest_head == 0) {
-                closest_head = 1; // first head is a safe bet here
+            FluxboxWindow& w = *(*win);
+
+            // check if the window is invisible
+            bool invisible = true;
+            int j;
+            for (j = 0; j < m_xinerama_num_heads; ++j) {
+                XineramaHeadInfo& hi = m_xinerama_headinfo[j];
+                if (RectangleUtil::overlapRectangles(hi, w)) {
+                    invisible = false;
+                    break;
+                }
+            }
+
+            if (invisible) { // get closest head and replace the (now invisible) cwindow
+                int closest_head = getHead(w.fbWindow());
+                if (closest_head == 0) {
+                    closest_head = 1; // first head is a safe bet here
+                }
+                w.placeWindow(closest_head);
             }
-            (*win)->placeWindow(closest_head);
         }
     }
 }
@@ -2003,10 +2081,7 @@ int BScreen::getHead(int x, int y) const {
 #ifdef XINERAMA
     if (hasXinerama()) {
         for (int i=0; i < m_xinerama_num_heads; i++) {
-            if (x >= m_xinerama_headinfo[i].x &&
-                    x < (m_xinerama_headinfo[i].x + m_xinerama_headinfo[i].width) &&
-                    y >= m_xinerama_headinfo[i].y &&
-                    y < (m_xinerama_headinfo[i].y + m_xinerama_headinfo[i].height)) {
+            if (RectangleUtil::insideBorder(m_xinerama_headinfo[i], x, y, 0)) {
                 return i+1;
             }
         }
@@ -2024,21 +2099,24 @@ int BScreen::getHead(const FbTk::FbWindow &win) const {
     if (hasXinerama()) {
 
         // cast needed to prevent win.x() become "unsigned int" which is bad
-        // since it might become negative
+        // since it might be negative
         int cx = win.x() + static_cast<int>(win.width() / 2);
         int cy = win.y() + static_cast<int>(win.height() / 2);
 
-        long dist = -1;
-
-        int i;
-        for (i = 0; i < m_xinerama_num_heads; ++i) {
-
-            XineramaHeadInfo& hi = m_xinerama_headinfo[i];
-            int d = calcSquareDistance(cx, cy, hi.x + (hi.width / 2), hi.y + (hi.height / 2));
-
-            if (dist == -1 || d < dist) { // found a closer head
-                head = i + 1;
-                dist = d;
+        head = getHead(cx, cy);
+        if ( head == 0 ) {
+            // if the center of the window is not on any head then select
+            // the head which center is nearest to the window center
+            long dist = -1;
+            int i;
+            for (i = 0; i < m_xinerama_num_heads; ++i) {
+                XineramaHeadInfo& hi = m_xinerama_headinfo[i];
+                int d = calcSquareDistance(cx, cy,
+                    hi.x() + (hi.width() / 2), hi.y() + (hi.height() / 2));
+                if (dist == -1 || d < dist) { // found a closer head
+                    head = i + 1;
+                    dist = d;
+                }
             }
         }
     }
@@ -2068,7 +2146,7 @@ int BScreen::getCurrHead() const {
 int BScreen::getHeadX(int head) const {
 #ifdef XINERAMA
     if (head == 0 || head > m_xinerama_num_heads) return 0;
-    return m_xinerama_headinfo[head-1].x;
+    return m_xinerama_headinfo[head-1].x();
 #else
     return 0;
 #endif // XINERAMA
@@ -2077,7 +2155,7 @@ int BScreen::getHeadX(int head) const {
 int BScreen::getHeadY(int head) const {
 #ifdef XINERAMA
     if (head == 0 || head > m_xinerama_num_heads) return 0;
-    return m_xinerama_headinfo[head-1].y;
+    return m_xinerama_headinfo[head-1].y();
 #else
     return 0;
 #endif // XINERAMA
@@ -2086,7 +2164,7 @@ int BScreen::getHeadY(int head) const {
 int BScreen::getHeadWidth(int head) const {
 #ifdef XINERAMA
     if (head == 0 || head > m_xinerama_num_heads) return width();
-    return m_xinerama_headinfo[head-1].width;
+    return m_xinerama_headinfo[head-1].width();
 #else
     return width();
 #endif // XINERAMA
@@ -2095,7 +2173,7 @@ int BScreen::getHeadWidth(int head) const {
 int BScreen::getHeadHeight(int head) const {
 #ifdef XINERAMA
     if (head == 0 || head > m_xinerama_num_heads) return height();
-    return m_xinerama_headinfo[head-1].height;
+    return m_xinerama_headinfo[head-1].height();
 #else
     return height();
 #endif // XINERAMA
