@@ -57,21 +57,9 @@
 #include "FbTk/KeyUtil.hh"
 #include "FbTk/MemFun.hh"
 
-//Use GNU extensions
-#ifndef	 _GNU_SOURCE
-#define	 _GNU_SOURCE
-#endif // _GNU_SOURCE
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif // HAVE_CONFIG_H
-
-#ifdef USE_GNOME
-#include "Gnome.hh"
-#endif // USE_GNOME
-#ifdef USE_NEWWMSPEC
+#ifdef USE_EWMH
 #include "Ewmh.hh"
-#endif // USE_NEWWMSPEC
+#endif // USE_EWMH
 #ifdef REMEMBER
 #include "Remember.hh"
 #endif // REMEMBER
@@ -87,7 +75,7 @@
 #ifdef SHAPE
 #include <X11/extensions/shape.h>
 #endif // SHAPE
-#ifdef HAVE_RANDR
+#if defined(HAVE_RANDR) || defined(HAVE_RANDR1_2)
 #include <X11/extensions/Xrandr.h>
 #endif // HAVE_RANDR
 
@@ -127,7 +115,9 @@
 #include <sys/stat.h>
 #endif // HAVE_SYS_STAT_H
 
+#ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
+#endif // HAVE_SYS_WAIT_H
 
 #include <iostream>
 #include <memory>
@@ -150,6 +140,8 @@ using namespace FbTk;
 
 namespace {
 
+const char RC_INIT_FILE[] = "init";
+
 Window last_bad_window = None;
 
 // *** NOTE: if you want to debug here the X errors are
@@ -176,11 +168,38 @@ int handleXErrors(Display *d, XErrorEvent *e) {
 //            kill(0, 2);
     }
 #endif // !DEBUG
-
     return False;
 }
 
 
+int handleXIOErrors(Display* d) {
+    cerr << "Fluxbox: XIOError: lost connection to display.\n";
+    exit(1);
+}
+
+
+
+class KeyReloadHelper {
+public:
+    void reload() {
+        Fluxbox* f = Fluxbox::instance();
+        Keys* k = (f ? f->keys() : 0);
+        if (k) {
+            XRefreshKeyboardMapping(&(this->xmapping));
+            FbTk::KeyUtil::instance().init();
+            k->regrab();
+        }
+    }
+
+    XMappingEvent xmapping;
+};
+
+KeyReloadHelper s_key_reloader;
+
+typedef FbTk::SimpleCommand<KeyReloadHelper> KeyReloadHelperCmd;
+typedef FbTk::SimpleCommand<Fluxbox> FluxboxCmd;
+
+
 
 /* functor to call a memberfunction with by a reference argument
    other places needs this helper as well it should be moved
@@ -210,10 +229,6 @@ struct CallMemFunWithRefArg : std::unary_function<Type, ResultType> {
         m_arg(arg),
         m_func(func) { }
 
-    ResultType operator()(Type p) const {
-        (p.*m_func)(m_arg);
-    }
-
     ResultType operator()(Type* p) const {
         (*p.*m_func)(m_arg);
     }
@@ -223,10 +238,17 @@ struct CallMemFunWithRefArg : std::unary_function<Type, ResultType> {
 };
 
 
+int s_randr_event_type = 0; ///< the type number of randr event
+int s_shape_eventbase = 0;  ///< event base for shape events
+bool s_have_shape = false ; ///< if shape is supported by server
+
+Atom s_kwm1_dockwindow;
+Atom s_kwm2_dockwindow;
+
+Fluxbox* s_singleton = 0;
+
 } // end anonymous
 
-//static singleton var
-Fluxbox *Fluxbox::s_singleton=0;
 
 Fluxbox::Fluxbox(int argc, char **argv,
                  const std::string& dpy_name,
@@ -238,7 +260,6 @@ Fluxbox::Fluxbox(int argc, char **argv,
       m_screen_rm(m_resourcemanager),
 
       m_RC_PATH(rc_path),
-      m_RC_INIT_FILE("init"),
       m_rc_ignoreborder(m_resourcemanager, false, "session.ignoreBorder", "Session.IgnoreBorder"),
       m_rc_pseudotrans(m_resourcemanager, false, "session.forcePseudoTransparency", "Session.forcePseudoTransparency"),
       m_rc_colors_per_channel(m_resourcemanager, 4,
@@ -266,8 +287,7 @@ Fluxbox::Fluxbox(int argc, char **argv,
       m_starting(true),
       m_restarting(false),
       m_shutdown(false),
-      m_server_grabs(0),
-      m_randr_event_type(0) {
+      m_server_grabs(0) {
 
     _FB_USES_NLS;
     if (s_singleton != 0)
@@ -282,55 +302,52 @@ Fluxbox::Fluxbox(int argc, char **argv,
     Display *disp = FbTk::App::instance()->display();
     // For KDE dock applets
     // KDE v1.x
-    m_kwm1_dockwindow = XInternAtom(disp,
+    s_kwm1_dockwindow = XInternAtom(disp,
                                     "KWM_DOCKWINDOW", False);
     // KDE v2.x
-    m_kwm2_dockwindow = XInternAtom(disp,
+    s_kwm2_dockwindow = XInternAtom(disp,
                                     "_KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR", False);
     // setup X error handler
-    XSetErrorHandler((XErrorHandler) handleXErrors);
+    XSetErrorHandler(handleXErrors);
+    XSetIOErrorHandler(handleXIOErrors);
 
-    //catch system signals
-    SignalHandler &sigh = SignalHandler::instance();
-    sigh.registerHandler(SIGSEGV, this);
-    sigh.registerHandler(SIGFPE, this);
-    sigh.registerHandler(SIGPIPE, this); // e.g. output sent to grep
-    sigh.registerHandler(SIGTERM, this);
-    sigh.registerHandler(SIGINT, this);
-    sigh.registerHandler(SIGCHLD, this);
-    sigh.registerHandler(SIGHUP, this);
-    sigh.registerHandler(SIGUSR1, this);
-    sigh.registerHandler(SIGUSR2, this);
     //
     // setup timer
     // This timer is used to we can issue a safe reconfig command.
     // Because when the command is executed we shouldn't do reconfig directly
     // because it could affect ongoing menu stuff so we need to reconfig in
     // the next event "round".
-    FbTk::RefCount<FbTk::Command<void> > reconfig_cmd(new FbTk::SimpleCommand<Fluxbox>(*this, &Fluxbox::timed_reconfigure));
-    m_reconfig_timer.setTimeout(0, 1);
-    m_reconfig_timer.setCommand(reconfig_cmd);
+    FluxboxCmd* reconfig_cmd = new FluxboxCmd(*this, &Fluxbox::timed_reconfigure);
+    m_reconfig_timer.setTimeout(1);
+    m_reconfig_timer.setCommand(FbTk::RefCount<FbTk::Command<void> >(reconfig_cmd));
     m_reconfig_timer.fireOnce(true);
 
+    // xmodmap and other tools send a lot of MappingNotify events under some
+    // circumstances ("keysym comma = comma semicolon" creates 4 or 5).
+    // reloading the keys-file for every one of them is unclever. we postpone
+    // the reload() via a timer.
+    KeyReloadHelperCmd* rh_cmd = 
+        new KeyReloadHelperCmd(s_key_reloader, &KeyReloadHelper::reload);
+    m_key_reload_timer.setTimeout(250 * FbTk::FbTime::IN_MILLISECONDS);
+    m_key_reload_timer.setCommand(FbTk::RefCount<FbTk::Command<void> >(rh_cmd));
+    m_key_reload_timer.fireOnce(true);
+
     if (xsync)
         XSynchronize(disp, True);
 
     s_singleton = this;
-    m_have_shape = false;
-    m_shape_eventbase = 0;
+
 #ifdef SHAPE
     int shape_err;
-    m_have_shape = XShapeQueryExtension(disp, &m_shape_eventbase, &shape_err);
+    s_have_shape = XShapeQueryExtension(disp, &s_shape_eventbase, &shape_err);
 #endif // SHAPE
 
-#ifdef HAVE_RANDR
-    // get randr event type
+#if defined(HAVE_RANDR) || defined(HAVE_RANDR1_2)
     int randr_error_base;
-    XRRQueryExtension(disp, &m_randr_event_type, &randr_error_base);
+    XRRQueryExtension(disp, &s_randr_event_type, &randr_error_base);
 #endif // HAVE_RANDR
 
     load_rc();
-
     grab();
 
     if (! XSupportsLocale())
@@ -365,7 +382,6 @@ Fluxbox::Fluxbox(int argc, char **argv,
         screens.push_back(i);
 
     // find out, on what "screens" fluxbox should run
-    // FIXME(php-coder): maybe it worths moving this code to main.cc, where command line is parsed?
     for (i = 1; i < m_argc; i++) {
         if (! strcmp(m_argv[i], "-screen")) {
             if ((++i) >= m_argc) {
@@ -384,7 +400,7 @@ Fluxbox::Fluxbox(int argc, char **argv,
             int scrnr = 0;
             FbTk::StringUtil::stringtok(vals, m_argv[i], ",:");
             for (vector<string>::iterator scrit = vals.begin();
-                 scrit != vals.end(); scrit++) {
+                 scrit != vals.end(); ++scrit) {
                 scrnr = atoi(scrit->c_str());
                 if (scrnr >= 0 && scrnr < ScreenCount(disp))
                     scrtmp.push_back(scrnr);
@@ -401,7 +417,7 @@ Fluxbox::Fluxbox(int argc, char **argv,
         BScreen *screen = new BScreen(m_screen_rm.lock(),
                                       std::string("session.screen") + sc_nr,
                                       std::string("session.Screen") + sc_nr,
-                                      screens[s], ::Layer::NUM_LAYERS);
+                                      screens[s], ::ResourceLayer::NUM_LAYERS);
 
         // already handled
         if (! screen->isScreenManaged()) {
@@ -421,12 +437,9 @@ Fluxbox::Fluxbox(int argc, char **argv,
 
     m_keyscreen = m_mousescreen = m_screen_list.front();
 
-#ifdef USE_NEWWMSPEC
-    addAtomHandler(new Ewmh()); // for Extended window manager atom support
-#endif // USE_NEWWMSPEC
-#ifdef USE_GNOME
-    addAtomHandler(new Gnome()); // for gnome 1 atom support
-#endif //USE_GNOME
+#ifdef USE_EWMH
+    addAtomHandler(new Ewmh());
+#endif // USE_EWMH
     // parse apps file after creating screens (so we can tell if it's a restart
     // for [startup] items) but before creating windows
     // this needs to be after ewmh and gnome, so state atoms don't get
@@ -512,8 +525,11 @@ void Fluxbox::initScreen(BScreen *screen) {
 
 
 void Fluxbox::eventLoop() {
+
     Display *disp = display();
+
     while (!m_shutdown) {
+
         if (XPending(disp)) {
             XEvent e;
             XNextEvent(disp, &e);
@@ -529,8 +545,9 @@ void Fluxbox::eventLoop() {
                 handleEvent(&e);
             }
         } else {
-            FbTk::Timer::updateTimers(ConnectionNumber(disp)); //handle all timers
+            FbTk::Timer::updateTimers(ConnectionNumber(disp));
         }
+
     }
 }
 
@@ -558,9 +575,11 @@ void Fluxbox::ungrab() {
 }
 
 void Fluxbox::handleEvent(XEvent * const e) {
+
     _FB_USES_NLS;
     m_last_event = *e;
 
+
     // it is possible (e.g. during moving) for a window
     // to mask all events to go to it
     if ((m_masked == e->xany.window) && m_masked_window) {
@@ -643,20 +662,12 @@ void Fluxbox::handleEvent(XEvent * const e) {
 
         if (! winclient) {
             BScreen *screen = 0;
-            int screen_num;
             XWindowAttributes attr;
             // find screen
             if (XGetWindowAttributes(display(),
                                      e->xmaprequest.window,
                                      &attr) && attr.screen != 0) {
-                screen_num = XScreenNumberOfScreen(attr.screen);
-
-                // find screen
-                ScreenList::iterator screen_it = find_if(m_screen_list.begin(),
-                                                         m_screen_list.end(),
-                                                         FbTk::CompareEqual<BScreen>(&BScreen::screenNumber, screen_num));
-                if (screen_it != m_screen_list.end())
-                    screen = *screen_it;
+                screen = findScreen(XScreenNumberOfScreen(attr.screen));
             }
             // try with parent if we failed to find screen num
             if (screen == 0)
@@ -680,21 +691,18 @@ void Fluxbox::handleEvent(XEvent * const e) {
         break;
     case UnmapNotify:
         handleUnmapNotify(e->xunmap);
-	break;
-    case MappingNotify:
-        // Update stored modifier mapping
-        fbdbg<<"MappingNotify"<<endl;
+        break;
+    case MappingNotify: // Update stored modifier mapping
 
         if (e->xmapping.request == MappingKeyboard
             || e->xmapping.request == MappingModifier) {
-            XRefreshKeyboardMapping(&e->xmapping);
-            FbTk::KeyUtil::instance().init(); // reinitialise the key utils
-            // reconfigure keys (if the mapping changes, they don't otherwise update
-            m_key->regrab();
+
+            s_key_reloader.xmapping = e->xmapping;
+            m_key_reload_timer.start();
         }
         break;
     case CreateNotify:
-	break;
+        break;
     case DestroyNotify: {
         WinClient *winclient = searchWindow(e->xdestroywindow.window);
         if (winclient != 0) {
@@ -716,7 +724,7 @@ void Fluxbox::handleEvent(XEvent * const e) {
         // most of them are handled in FluxboxWindow::handleEvent
         // but some special cases like ewmh propertys needs to be checked
         for (AtomHandlerContainerIt it= m_atomhandler.begin();
-             it != m_atomhandler.end(); it++) {
+             it != m_atomhandler.end(); ++it) {
             if ( (*it)->propertyNotify(*winclient, e->xproperty.atom))
                 break;
         }
@@ -796,8 +804,8 @@ void Fluxbox::handleEvent(XEvent * const e) {
         break;
     default: {
 
-#ifdef HAVE_RANDR
-        if (e->type == m_randr_event_type) {
+#if defined(HAVE_RANDR) || defined(HAVE_RANDR1_2)
+        if (e->type == s_randr_event_type) {
 #ifdef HAVE_RANDR1_2
             XRRUpdateConfiguration(e);
 #endif
@@ -885,151 +893,95 @@ void Fluxbox::handleClientMessage(XClientMessageEvent &ce) {
         // note: we dont need screen nor winclient to be non-null,
         // it's up to the atomhandler to check that
         for (AtomHandlerContainerIt it= m_atomhandler.begin();
-             it != m_atomhandler.end(); it++) {
+             it != m_atomhandler.end(); ++it) {
             (*it)->checkClientMessage(ce, screen, winclient);
         }
 
     }
 }
 
-/// handle system signals
-void Fluxbox::handleSignal(int signum) {
-    _FB_USES_NLS;
+void Fluxbox::windowDied(Focusable &focusable) {
+    FluxboxWindow *fbwin = focusable.fbwindow();
 
-    static int re_enter = 0;
+    STLUtil::forAllIf(m_atomhandler, mem_fun(&AtomHandler::update),
+        CallMemFunWithRefArg<AtomHandler, FluxboxWindow&, void>(&AtomHandler::updateFrameClose, *focusable.fbwindow()));
 
-    switch (signum) {
-    case SIGCHLD: // we don't want the child process to kill us
-        // more than one process may have terminated
-        while (waitpid(-1, 0, WNOHANG | WUNTRACED) > 0);
-        break;
-    case SIGHUP:
-        restart();
-        break;
-    case SIGUSR1:
-        load_rc();
-        break;
-    case SIGUSR2:
-        reconfigure();
-        break;
-    case SIGSEGV:
-        abort();
-        break;
-    case SIGFPE:
-    case SIGINT:
-    case SIGPIPE:
-    case SIGTERM:
-        shutdown();
-        break;
-    default:
-        fprintf(stderr,
-                _FB_CONSOLETEXT(BaseDisplay, SignalCaught, "%s:      signal %d caught\n", "signal catch debug message. Include %s for Command<void> and %d for signal number").c_str(),
-                m_argv[0], signum);
+    // make sure each workspace get this
+    BScreen &scr = focusable.screen();
+    scr.removeWindow(fbwin);
+    if (FocusControl::focusedFbWindow() == fbwin)
+        FocusControl::setFocusedFbWindow(0);
+}
 
-        if (! m_starting && ! re_enter) {
-            re_enter = 1;
-            cerr<<_FB_CONSOLETEXT(BaseDisplay, ShuttingDown, "Shutting Down\n", "Quitting because of signal, end with newline");
-            shutdown();
-        }
+void Fluxbox::clientDied(Focusable &focusable) {
+    WinClient &client = dynamic_cast<WinClient &>(focusable);
 
+    STLUtil::forAllIf(m_atomhandler, mem_fun(&AtomHandler::update),
+            CallMemFunWithRefArg<AtomHandler, WinClient&, void>(&AtomHandler::updateClientClose, client));
 
-        cerr<<_FB_CONSOLETEXT(BaseDisplay, Aborting, "Aborting... dumping core\n", "Aboring and dumping core, end with newline");
-        abort();
-        break;
+    BScreen &screen = client.screen();
+
+    // At this point, we trust that this client is no longer in the
+    // client list of its frame (but it still has reference to the frame)
+    // We also assume that any remaining active one is the last focused one
+
+    // This is where we revert focus on window close
+    // NOWHERE ELSE!!!
+    if (FocusControl::focusedWindow() == &client) {
+        FocusControl::unfocusWindow(client);
+        // make sure nothing else uses this window before focus reverts
+        FocusControl::setFocusedWindow(0);
+    } else if (FocusControl::expectingFocus() == &client) {
+        FocusControl::setExpectingFocus(0);
+        revertFocus();
+    }
+
+    screen.removeClient(client);
+}
+
+void Fluxbox::windowWorkspaceChanged(FluxboxWindow &win) {
+    STLUtil::forAllIf(m_atomhandler, mem_fun(&AtomHandler::update),
+        CallMemFunWithRefArg<AtomHandler, FluxboxWindow&, void>(&AtomHandler::updateWorkspace, win));
+}
+
+void Fluxbox::windowStateChanged(FluxboxWindow &win) {
+    STLUtil::forAllIf(m_atomhandler, mem_fun(&AtomHandler::update),
+        CallMemFunWithRefArg<AtomHandler, FluxboxWindow&, void>(&AtomHandler::updateState, win));
+
+    // if window changed to iconic state
+    // add to icon list
+    if (win.isIconic()) {
+        win.screen().addIcon(&win);
+        Workspace *space = win.screen().getWorkspace(win.workspaceNumber());
+        if (space != 0)
+            space->removeWindow(&win, true);
+    }
+
+    if (win.isStuck()) {
+        // if we're sticky then reassociate window
+        // to all workspaces
+        BScreen &scr = win.screen();
+        if (scr.currentWorkspaceID() != win.workspaceNumber())
+            scr.reassociateWindow(&win, scr.currentWorkspaceID(), true);
     }
 }
 
-
-void Fluxbox::update(FbTk::Subject *changedsub) {
-    //TODO: fix signaling, this does not look good
-    FluxboxWindow *fbwin = 0;
-    WinClient *client = 0;
-
-    if (typeid(*changedsub) == typeid(FluxboxWindow::WinSubject)) {
-        FluxboxWindow::WinSubject *winsub = dynamic_cast<FluxboxWindow::WinSubject *>(changedsub);
-        fbwin = &winsub->win();
-    } else if (typeid(*changedsub) == typeid(Focusable::FocusSubject)) {
-        Focusable::FocusSubject *winsub = dynamic_cast<Focusable::FocusSubject *>(changedsub);
-        fbwin = winsub->win().fbwindow();
-        if (typeid(winsub->win()) == typeid(WinClient))
-            client = dynamic_cast<WinClient *>(&winsub->win());
-    }
-
-    if (fbwin && &fbwin->stateSig() == changedsub) { // state signal
-        STLUtil::forAllIf(m_atomhandler, mem_fun(&AtomHandler::update),
-                CallMemFunWithRefArg<AtomHandler, FluxboxWindow&, void>(&AtomHandler::updateState, *fbwin));
-
-        // if window changed to iconic state
-        // add to icon list
-        if (fbwin->isIconic()) {
-            fbwin->screen().addIcon(fbwin);
-            Workspace *space = fbwin->screen().getWorkspace(fbwin->workspaceNumber());
-            if (space != 0)
-                space->removeWindow(fbwin, true);
-        }
-
-        if (fbwin->isStuck()) {
-            // if we're sticky then reassociate window
-            // to all workspaces
-            BScreen &scr = fbwin->screen();
-            if (scr.currentWorkspaceID() != fbwin->workspaceNumber()) {
-                scr.reassociateWindow(fbwin,
-                                      scr.currentWorkspaceID(),
-                                      true);
-            }
-        }
-    } else if (fbwin && &fbwin->layerSig() == changedsub) { // layer signal
-        STLUtil::forAllIf(m_atomhandler, mem_fun(&AtomHandler::update),
-            CallMemFunWithRefArg<AtomHandler, FluxboxWindow&, void>(&AtomHandler::updateLayer, *fbwin));
-    } else if (fbwin && &fbwin->dieSig() == changedsub) { // window death signal
-        STLUtil::forAllIf(m_atomhandler, mem_fun(&AtomHandler::update),
-            CallMemFunWithRefArg<AtomHandler, FluxboxWindow&, void>(&AtomHandler::updateFrameClose, *fbwin));
-
-        // make sure each workspace get this
-        BScreen &scr = fbwin->screen();
-        scr.removeWindow(fbwin);
-        if (FocusControl::focusedFbWindow() == fbwin)
-            FocusControl::setFocusedFbWindow(0);
-    } else if (fbwin && &fbwin->workspaceSig() == changedsub) {  // workspace signal
-        STLUtil::forAllIf(m_atomhandler, mem_fun(&AtomHandler::update),
-            CallMemFunWithRefArg<AtomHandler, FluxboxWindow&, void>(&AtomHandler::updateWorkspace, *fbwin));
-    } else if (client && &client->dieSig() == changedsub) { // client death
-        STLUtil::forAllIf(m_atomhandler, mem_fun(&AtomHandler::update),
-            CallMemFunWithRefArg<AtomHandler, WinClient&, void>(&AtomHandler::updateClientClose, *client));
-
-        BScreen &screen = client->screen();
-
-        // At this point, we trust that this client is no longer in the
-        // client list of its frame (but it still has reference to the frame)
-        // We also assume that any remaining active one is the last focused one
-
-        // This is where we revert focus on window close
-        // NOWHERE ELSE!!!
-        if (FocusControl::focusedWindow() == client) {
-            FocusControl::unfocusWindow(*client);
-            // make sure nothing else uses this window before focus reverts
-            FocusControl::setFocusedWindow(0);
-        } else if (FocusControl::expectingFocus() == client) {
-            FocusControl::setExpectingFocus(0);
-            revertFocus();
-        }
-
-        screen.removeClient(*client);
-    }
+void Fluxbox::windowLayerChanged(FluxboxWindow &win) {
+    STLUtil::forAllIf(m_atomhandler, mem_fun(&AtomHandler::update),
+        CallMemFunWithRefArg<AtomHandler, FluxboxWindow&, void>(&AtomHandler::updateLayer, win));
 }
 
 void Fluxbox::attachSignals(FluxboxWindow &win) {
-    win.hintSig().attach(this);
-    win.stateSig().attach(this);
-    win.workspaceSig().attach(this);
-    win.layerSig().attach(this);
-    win.dieSig().attach(this);
+    join(win.stateSig(), FbTk::MemFun(*this, &Fluxbox::windowStateChanged));
+    join(win.workspaceSig(), FbTk::MemFun(*this, &Fluxbox::windowWorkspaceChanged));
+    join(win.layerSig(), FbTk::MemFun(*this, &Fluxbox::windowLayerChanged));
+    join(win.dieSig(), FbTk::MemFun(*this, &Fluxbox::windowDied));
     STLUtil::forAll(m_atomhandler,
             CallMemFunWithRefArg<AtomHandler, FluxboxWindow&, void>(&AtomHandler::setupFrame, win));
 }
 
 void Fluxbox::attachSignals(WinClient &winclient) {
-    winclient.dieSig().attach(this);
+    join(winclient.dieSig(), FbTk::MemFun(*this, &Fluxbox::clientDied));
     STLUtil::forAll(m_atomhandler,
             CallMemFunWithRefArg<AtomHandler, WinClient&, void>(&AtomHandler::setupClient, winclient));
 }
@@ -1051,7 +1003,7 @@ AtomHandler* Fluxbox::getAtomHandler(const string &name) {
     if ( name != "" ) {
 
         AtomHandlerContainerIt it;
-        for (it = m_atomhandler.begin(); it != m_atomhandler.end(); it++) {
+        for (it = m_atomhandler.begin(); it != m_atomhandler.end(); ++it) {
             if (name == (*it)->getName())
                 return *it;
         }
@@ -1122,18 +1074,30 @@ void Fluxbox::restart(const char *prog) {
     }
 }
 
-/// prepares fluxbox for a shutdown
-void Fluxbox::shutdown() {
+// prepares fluxbox for a shutdown. when x_wants_down is != 0 we assume that
+// the xserver is about to shutdown or is in the midst of shutting down
+// already. trying to cleanup over a shaky xserver connection is pointless and
+// might lead to hangups.
+void Fluxbox::shutdown(int x_wants_down) {
     if (m_shutdown)
         return;
 
+    Display *dpy = FbTk::App::instance()->display();
     m_shutdown = true;
 
-    XSetInputFocus(FbTk::App::instance()->display(), PointerRoot, None, CurrentTime);
+#ifdef HAVE_ALARM
+    // give ourself 2 seconds (randomly picked randon number) to shutdown
+    // and then try to reenter signal handling. a bad race condition might
+    // lead to an inifite loop and this is some kind of last resort
+    alarm(2);
+#endif
 
-    STLUtil::forAll(m_screen_list, mem_fun(&BScreen::shutdown));
+    XSetInputFocus(dpy, PointerRoot, None, CurrentTime);
 
-    sync(false);
+    if (x_wants_down == 0) {
+        STLUtil::forAll(m_screen_list, mem_fun(&BScreen::shutdown));
+        sync(false);
+    }
 }
 
 /// saves resources
@@ -1187,7 +1151,7 @@ void Fluxbox::save_rc() {
 /// @return filename of resource file
 string Fluxbox::getRcFilename() {
     if (m_rc_file.empty())
-        return getDefaultDataFilename(m_RC_INIT_FILE);
+        return getDefaultDataFilename(RC_INIT_FILE);
     return m_rc_file;
 }
 
@@ -1219,9 +1183,7 @@ void Fluxbox::load_rc() {
 
     FbTk::Transparent::usePseudoTransparent(*m_rc_pseudotrans);
 
-    if (!m_rc_slitlistfile->empty()) {
-        *m_rc_slitlistfile = StringUtil::expandFilename(*m_rc_slitlistfile);
-    } else {
+    if (m_rc_slitlistfile->empty()) {
         string filename = getDefaultDataFilename("slitlist");
         m_rc_slitlistfile.setFromString(filename.c_str());
     }
@@ -1267,7 +1229,7 @@ void Fluxbox::load_rc(BScreen &screen) {
         StringUtil::removeFirstWhitespace(values);
         StringUtil::stringtok<BScreen::WorkspaceNames>(names, values, ",");
         BScreen::WorkspaceNames::iterator it;
-        for(it = names.begin(); it != names.end(); it++) {
+        for(it = names.begin(); it != names.end(); ++it) {
             if (!(*it).empty() && (*it) != "")
             screen.addWorkspaceName((*it).c_str());
         }
@@ -1288,6 +1250,7 @@ void Fluxbox::load_rc(BScreen &screen) {
 }
 
 void Fluxbox::reconfigure() {
+    m_key_reload_timer.stop();
     load_rc();
     m_reconfigure_wait = true;
     m_reconfig_timer.start();
@@ -1359,7 +1322,7 @@ bool Fluxbox::validateClient(const WinClient *client) const {
     WinClientMap::const_iterator it =
         find_if(m_window_search.begin(),
                 m_window_search.end(),
-                Compose(bind2nd(equal_to<WinClient *>(), client),
+                Compose(bind2nd(equal_to<const WinClient *>(), client),
                         Select2nd<WinClientMap::value_type>()));
     return it != m_window_search.end();
 }
@@ -1394,7 +1357,7 @@ void Fluxbox::focusedWindowChanged(BScreen &screen,
                                    WinClient* client) {
 
     for (AtomHandlerContainerIt it= m_atomhandler.begin();
-         it != m_atomhandler.end(); it++) {
+         it != m_atomhandler.end(); ++it) {
         (*it)->updateFocusedWindow(screen, client ? client->window() : 0 );
     }
 }
@@ -1404,3 +1367,17 @@ void Fluxbox::workspaceAreaChanged(BScreen &screen) {
             CallMemFunWithRefArg<AtomHandler, BScreen&, void>(&AtomHandler::updateWorkarea, screen));
 }
 
+bool Fluxbox::haveShape() const {
+    return s_have_shape;
+}
+
+int Fluxbox::shapeEventbase() const {
+    return s_shape_eventbase;
+}
+
+Fluxbox* Fluxbox::instance() {
+    return s_singleton;
+}
+
+
+
