@@ -26,19 +26,10 @@
 
 #include "../src/defaults.hh"
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif // HAVE_CONFIG_H
-
 #ifdef HAVE_SIGNAL_H
 #include <signal.h>
 #endif // HAVE_SIGNAL_H
 
-//use GNU extensions
-#ifndef         _GNU_SOURCE
-#define         _GNU_SOURCE
-#endif // _GNU_SOURCE
-
 #ifdef HAVE_CSTRING
   #include <cstring>
 #else
@@ -64,7 +55,6 @@ using std::set;
 using std::map;
 using std::list;
 using std::exit;
-using std::getenv;
 
 string read_file(const string& filename);
 void write_file(const string& filename, const string &contents);
@@ -615,7 +605,7 @@ int main(int argc, char **argv) {
     }
 
     if (rc_filename.empty())
-        rc_filename = getenv("HOME") + string("/.fluxbox/init");
+        rc_filename = FbTk::StringUtil::expandFilename("~/.fluxbox/init");
 
     FbTk::ResourceManager resource_manager(rc_filename.c_str(),false);
     if (!resource_manager.load(rc_filename.c_str())) {
@@ -651,11 +641,11 @@ int main(int argc, char **argv) {
         resource_manager.save(rc_filename.c_str(), rc_filename.c_str());
         save_all_files();
 
-#ifdef HAVE_SIGNAL_H
+#if defined(HAVE_SIGNAL_H) && !defined(_WIN32)
         // if we were given a fluxbox pid, send it a reconfigure signal
         if (fb_pid > 0)
             kill(fb_pid, SIGUSR2);
-#endif // HAVE_SIGNAL_H
+#endif // defined(HAVE_SIGNAL_H) && !defined(_WIN32)
 
     }
 
